@@ -1,11 +1,14 @@
 import abc
 import asyncio
 import inspect
+import pprint
+import textwrap
 
 from aiozmq import interface
 from types import MethodType
 
 from .packer import _Packer
+from .log import logger
 
 
 class Error(Exception):
@@ -15,10 +18,16 @@ class Error(Exception):
 class GenericError(Error):
     """Error used for all untranslated exceptions from rpc method calls."""
 
-    def __init__(self, exc_type, args):
-        super().__init__(exc_type, args)
+    def __init__(self, exc_type, args, exc_repr):
+        super().__init__(exc_type, args, exc_repr)
         self.exc_type = exc_type
         self.arguments = args
+        self.exc_repr = exc_repr
+
+    def __repr__(self):
+        return '<Generic RPC Error {}{}: {}>'.format(self.exc_type,
+                                                     self.arguments,
+                                                     self.exc_repr)
 
 
 class NotFoundError(Error, LookupError):
@@ -30,7 +39,7 @@ class ParametersError(Error, ValueError):
     be validated against their annotations."""
 
 
-class ServiceClosedError(Exception):
+class ServiceClosedError(Error):
     """RPC Service has been closed."""
 
 
@@ -142,11 +151,13 @@ class _BaseProtocol(interface.ZmqProtocol):
 
 class _BaseServerProtocol(_BaseProtocol):
 
-    def __init__(self, loop, handler, *, translation_table=None):
+    def __init__(self, loop, handler, *,
+                 translation_table=None, log_exceptions=False):
         super().__init__(loop, translation_table=translation_table)
-        self.handler = handler
         if not isinstance(handler, AbstractHandler):
             raise TypeError('handler should implement AbstractHandler ABC')
+        self.handler = handler
+        self.log_exceptions = log_exceptions
 
     def dispatch(self, name):
         if not name:
@@ -176,7 +187,7 @@ class _BaseServerProtocol(_BaseProtocol):
                 raise NotFoundError(name)
             return func
 
-    def _check_func_arguments(self, func, args, kwargs):
+    def check_args(self, func, args, kwargs):
         """Utility function for validating function arguments
 
         Returns validated (args, kwargs, return annotation) tuple
@@ -204,3 +215,15 @@ class _BaseServerProtocol(_BaseProtocol):
             if sig.return_annotation is not sig.empty:
                 return bargs.args, bargs.kwargs, sig.return_annotation
             return bargs.args, bargs.kwargs, None
+
+    def try_log(self, fut, name, args, kwargs):
+        try:
+            fut.result()
+        except Exception:
+            if self.log_exceptions:
+                logger.exception(textwrap.dedent("""\
+                    An exception from method %r call has been occurred.
+                    args = %s
+                    kwargs = %s
+                    """),
+                    name, pprint.pformat(args), pprint.pformat(kwargs))  # noqa
