@@ -3,7 +3,7 @@ import zmq
 
 from functools import partial
 
-from aiozmq.log import logger
+from .log import logger
 
 from .base import (
     NotFoundError,
@@ -20,7 +20,19 @@ from .util import (
 @asyncio.coroutine
 def connect_pipeline(*, connect=None, bind=None, loop=None,
                      translation_table=None):
-    """A coroutine that creates and connects/binds Pipeline client instance."""
+    """A coroutine that creates and connects/binds Pipeline client instance.
+
+    Usually for this function you need to use *connect* parameter, but
+    ZeroMQ does not forbid to use *bind*.
+
+    translation_table -- an optional table for custom value translators.
+
+    loop --  an optional parameter to point
+    ZmqEventLoop instance.  If loop is None then default
+    event loop will be given by asyncio.get_event_loop() call.
+
+    Returns PipelineClient instance.
+    """
     if loop is None:
         loop = asyncio.get_event_loop()
 
@@ -32,14 +44,32 @@ def connect_pipeline(*, connect=None, bind=None, loop=None,
 
 @asyncio.coroutine
 def serve_pipeline(handler, *, connect=None, bind=None, loop=None,
-                   translation_table=None):
-    """A coroutine that creates and connects/binds Pipeline server instance."""
+                   translation_table=None, log_exceptions=False):
+    """A coroutine that creates and connects/binds Pipeline server instance.
+
+    Usually for this function you need to use *bind* parameter, but
+    ZeroMQ does not forbid to use *connect*.
+
+    handler -- an object which processes incoming pipeline calls.
+    Usually you like to pass AttrHandler instance.
+
+    log_exceptions -- log exceptions from remote calls if True.
+
+    translation_table -- an optional table for custom value translators.
+
+    loop -- an optional parameter to point
+       ZmqEventLoop instance.  If loop is None then default
+       event loop will be given by asyncio.get_event_loop() call.
+
+    Returns Service instance.
+    """
     if loop is None:
         loop = asyncio.get_event_loop()
 
     trans, proto = yield from loop.create_zmq_connection(
         lambda: _ServerProtocol(loop, handler,
-                                translation_table=translation_table),
+                                translation_table=translation_table,
+                                log_exceptions=log_exceptions),
         zmq.PULL, connect=connect, bind=bind)
     return Service(loop, proto)
 
@@ -81,36 +111,27 @@ class _ServerProtocol(_BaseServerProtocol):
         try:
             name = bname.decode('utf-8')
             func = self.dispatch(name)
-            args, kwargs, ret_ann = self._check_func_arguments(
-                func, args, kwargs)
+            args, kwargs, ret_ann = self.check_args(func, args, kwargs)
         except (NotFoundError, ParametersError) as exc:
             fut = asyncio.Future(loop=self.loop)
-            fut.add_done_callback(partial(self.process_call_result,
-                                          name=name))
             fut.set_exception(exc)
         else:
             if asyncio.iscoroutinefunction(func):
                 fut = asyncio.async(func(*args, **kwargs), loop=self.loop)
-                fut.add_done_callback(partial(self.process_call_result,
-                                              name=name))
             else:
                 fut = asyncio.Future(loop=self.loop)
-                fut.add_done_callback(partial(self.process_call_result,
-                                              name=name))
                 try:
                     fut.set_result(func(*args, **kwargs))
                 except Exception as exc:
                     fut.set_exception(exc)
+        fut.add_done_callback(partial(self.process_call_result,
+                                      name=name, args=args, kwargs=kwargs))
 
-    def process_call_result(self, fut, *, name):
+    def process_call_result(self, fut, *, name, args, kwargs):
         try:
             if fut.result() is not None:
                 logger.warning("Pipeline handler %r returned not None", name)
-        except Exception as exc:
-            self.loop.call_exception_handler({
-                'message': 'Call to {!r} caused error: {!r}'.format(name, exc),
-                'exception': exc,
-                'future': fut,
-                'protocol': self,
-                'transport': self.transport,
-                })
+        except (NotFoundError, ParametersError) as exc:
+            logger.exception("Call to %r caused error: %r", name, exc)
+        except Exception:
+            self.try_log(fut, name, args, kwargs)
